﻿//-----------------------------------------------------------------------------
// <copyright file="MMM8x8Dize.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Dize for MMM8x8.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Cube
{
    using System;
    using System.Linq;
    using System.Threading;
    using MMM8x8Snake;
    using MMM8x8Utils;

    /// <summary>
    /// Dize for MMM8x8.
    /// </summary>
    public class MMM8x8Dize
    {
        /// <summary>
        /// Amount of numbers to show between one dize roll.
        /// </summary>
        private const int NumbersToShowOnOneDizeRoll = 30;

        /// <summary>
        /// Pause between two shown numbers in one dize roll.
        /// </summary>
        private const int PauseBetweenRolls = 50;

        /// <summary>
        /// Available numbers on the dize.
        /// </summary>
        private static readonly int[] AvailableNumbers = new int[] { 1, 2, 3, 4, 5, 6 };

        #region dize number initialization
        /// <summary>
        /// Two dimensional bool array for the 1.
        /// </summary>
        private static readonly bool[,] DizeNumber1 = new bool[,]
        {
            { false, false, false, false, true,  false, false, false }, 
            { false, false, false, true,  true,  false, false, false }, 
            { false, false, true,  false, true,  false, false, false }, 
            { false, false, false, false, true,  false, false, false }, 
            { false, false, false, false, true,  false, false, false }, 
            { false, false, false, false, true,  false, false, false }, 
            { false, false, true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, false, false, false }
        };

        /// <summary>
        /// Two dimensional bool array for the 2.
        /// </summary>
        private static readonly bool[,] DizeNumber2 = new bool[,]
        {
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, true,  false, false, false, false, false, false }, 
            { false, true,  false, false, false, false, false, false }, 
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, false, false, false }
        };

        /// <summary>
        /// Two dimensional bool array for the 3.
        /// </summary>
        private static readonly bool[,] DizeNumber3 = new bool[,]
        {
            { false, true,  true,  true,  true,  true,  false, false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, true,  true,  true,  true,  true,  false, false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, true,  true,  true,  true,  true,  false, false }
        };

        /// <summary>
        /// Two dimensional bool array for the 4.
        /// </summary>
        private static readonly bool[,] DizeNumber4 = new bool[,]
        {
            { false, false, false, false, true,  true,  false, false }, 
            { false, false, false, true,  false, true,  false, false }, 
            { false, false, true,  false, false, true,  false, false }, 
            { false, true,  false, false, false, true,  false, false }, 
            { true,  true,  true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, true,  false, false }, 
            { false, false, false, false, false, true,  false, false }, 
            { false, false, false, false, false, true,  false, false }
        };

        /// <summary>
        /// Two dimensional bool array for the 5.
        /// </summary>
        private static readonly bool[,] DizeNumber5 = new bool[,]
        {
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, true,  false, false, false, false, false, false }, 
            { false, true,  false, false, false, false, false, false }, 
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, false, false, false, false, false, true,  false }, 
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, false, false, false }
        };

        /// <summary>
        /// Two dimensional bool array for the 6.
        /// </summary>
        private static readonly bool[,] DizeNumber6 = new bool[,]
        {
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, true,  false, false, false, false, false, false }, 
            { false, true,  false, false, false, false, false, false }, 
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, true,  false, false, false, false, true,  false }, 
            { false, true,  false, false, false, false, true,  false }, 
            { false, true,  true,  true,  true,  true,  true,  false }, 
            { false, false, false, false, false, false, false, false }
        };
        #endregion

        /// <summary>
        /// MMM8x8 for device communication.
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// The available numbers on the dize get shuffled. During the roll
        /// the numbers get displayed on the MMM8x8 in this order.
        /// </summary>
        private int[] shuffledDizeNumbers;

        /// <summary>
        /// Last shown index index of the shuffledDizeNumbers array on the MMM8x8.
        /// </summary>
        private int lastShownShuffledDizeNumberIndex;
        
        /// <summary>
        /// Initializes a new instance of the MMM8x8Dize class.
        /// </summary>
        /// <param name="mmm8x8">MMM8x8 as ComPortDevice.</param>
        public MMM8x8Dize(ComPortDevice mmm8x8)
        {
            this.device = mmm8x8;

            this.lastShownShuffledDizeNumberIndex = -1;
        }

        /// <summary>
        /// Rolls the dize. A roll means n (MMM8x8Dize.NumbersToShowOnOneDizeRoll)
        /// shown numbers on the MMM8x8.
        /// </summary>
        public void Roll()
        {
            this.shuffledDizeNumbers = this.GetRandomizedDizeNumbers();
            for (int i = 0; i < MMM8x8Dize.NumbersToShowOnOneDizeRoll; i++)
            {
                this.ShowRandomNumberOnDevive();
                Thread.Sleep(MMM8x8Dize.PauseBetweenRolls);
            }
        }

        /// <summary>
        /// Randomizes the content of a passed array.
        /// </summary>
        /// <typeparam name="T">Type of the array.</typeparam>
        /// <param name="array">Array to randomize.</param>
        /// <returns>Randomized array.</returns>
        private static T[] RandomizeArray<T>(T[] array)
        {
            return array.OrderBy(n => Guid.NewGuid()).ToArray();
        }

        /// <summary>
        /// Gets a randomized array of the available dize numbers.
        /// </summary>
        /// <returns>Randomized array of the available dize numbers.</returns>
        private int[] GetRandomizedDizeNumbers()
        {
            return MMM8x8Dize.RandomizeArray<int>(MMM8x8Dize.AvailableNumbers);
        }

        /// <summary>
        /// Shows a random number on the MMM8x8.
        /// </summary>
        private void ShowRandomNumberOnDevive()
        {
            var nextIndex = ++this.lastShownShuffledDizeNumberIndex % MMM8x8Dize.AvailableNumbers.Length;
            var numberToShow = this.shuffledDizeNumbers[nextIndex];

            this.ShowNumberOnDevice(numberToShow);
        }

        /// <summary>
        /// Shows a passed number on the MMM8x8.
        /// </summary>
        /// <param name="number">Number to show.</param>
        private void ShowNumberOnDevice(int number)
        {
            var dizeNumberArray = this.GetDizeNumberArray(number);

            var mmm8x8LedMatrix = new MMM8x8LedMatrix();
            for (int x = 0; x < MMM8x8LedMatrix.FieldWidth; x++)
            {
                for (int y = 0; y < MMM8x8LedMatrix.FieldHeight; y++)
                {
                    mmm8x8LedMatrix[x, y] = dizeNumberArray[x, y];
                }
            }

            this.device.SendCommandToDevice((byte)MMM8x8.Commands.SetAllLedsCommand, mmm8x8LedMatrix.GetBytes());
        }

        /// <summary>
        /// Gets a passed number as two dimensional bool array which fits
        /// the two dimensional LED matrix of the MMM8x8.
        /// </summary>
        /// <param name="number">Number to convert.</param>
        /// <returns>Two dimensional bool array.</returns>
        private bool[,] GetDizeNumberArray(int number)
        {
            if (Array.IndexOf(MMM8x8Dize.AvailableNumbers, number) == -1)
            {
                throw new ArgumentException("The passed number is not available.");
            }

            switch (number)
            {
                case 1:
                    return MMM8x8Dize.DizeNumber1;

                case 2:
                    return MMM8x8Dize.DizeNumber2;

                case 3:
                    return MMM8x8Dize.DizeNumber3;

                case 4:
                    return MMM8x8Dize.DizeNumber4;

                case 5:
                    return MMM8x8Dize.DizeNumber5;

                case 6:
                    return MMM8x8Dize.DizeNumber6;

                default:
                    throw new ArgumentException("The passed number is not available.");
            }
        }
    }
}
