﻿//-----------------------------------------------------------------------------
// <copyright file="MainWindow.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Main window of the application.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Cube
{
    using System;
    using System.ComponentModel;
    using System.IO;
    using System.IO.Ports;
    using System.Windows.Forms;
    using MMM8x8Snake;
    using MMM8x8Utils;

    /// <summary>
    /// Main window of the application.
    /// </summary>
    public partial class MainWindow : Form
    {
        /// <summary>
        /// MMM8x8 as ComPortDevice.
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Load-method of the main window.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void MainWindow_Load(object sender, EventArgs e)
        {
            this.cbCOM.Items.AddRange(SerialPort.GetPortNames());
            if (this.cbCOM.Items.Count > 0)
            {
                this.cbCOM.SelectedIndex = 0;
            }
        }

        /// <summary>
        /// Initializes a dize roll in the GUI.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void BtnStartDizeRoll_Click(object sender, EventArgs e)
        {
            try
            {
                this.CreateComConnection();
            }
            catch (IOException ex)
            {
                MessageBox.Show(
                    string.Format("FEHLER: {0}", ex.Message),
                    "Fehler beim Verbinden mit MMM8x8",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                this.CloseDeviceConnection();
                return;
            }

            var rollDiceWorker = new BackgroundWorker();
            rollDiceWorker.DoWork += new DoWorkEventHandler(this.RollDiceWorker_DoWork);
            rollDiceWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(this.RollDiceWorker_RunWorkerCompleted);

            this.btnStartDizeRoll.Enabled = false;
            this.cbCOM.Enabled = false;

            rollDiceWorker.RunWorkerAsync();
        }

        /// <summary>
        /// Creates and checks the COM connection to the MMM8x8.
        /// </summary>
        private void CreateComConnection()
        {
            if (this.cbCOM.SelectedIndex == -1)
            {
                throw new IOException("Es wurde kein COM ausgewählt.");
            }

            var com = this.cbCOM.Items[this.cbCOM.SelectedIndex].ToString();
            SerialPort serialPort = new SerialPort(
                com,
                MMM8x8.BaudRate,
                MMM8x8.Parity,
                MMM8x8.Databits,
                MMM8x8.StopBits);

            try
            {
                serialPort.ReadTimeout = MMM8x8.ReadTimeout;
                serialPort.WriteTimeout = MMM8x8.WriteTimeout;
                serialPort.Open();
            }
            catch (UnauthorizedAccessException)
            {
                throw new IOException(string.Format("Mit {0} wurde bereits eine Verbindung hergestellt.", com));
            }
            catch (IOException)
            {
                throw new IOException(string.Format("Mit {0} ist keine Verbindung möglich.", com));
            }

            this.device = new ComPortDevice(serialPort);

            // check connection (turn all leds off)
            try
            {
                var mmm8x8LedMatrix = new MMM8x8LedMatrix();
                this.device.SendCommandToDevice((byte)MMM8x8.Commands.SetAllLedsCommand, mmm8x8LedMatrix.GetBytes());
            }
            catch (TimeoutException)
            {
                throw new IOException(string.Format("Mit {0} ist keine Verbindung zu einem MMM8x8 möglich.", com));
            }
        }

        /// <summary>
        /// Rolls the dize.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void RollDiceWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            var dize = new MMM8x8Dize(this.device);
            dize.Roll();
        }

        /// <summary>
        /// Manages processes after the dize roll.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void RollDiceWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(
                    "Während der Ansteuerung des MMM8x8 ist ein Fehler aufgetreten.",
                    "Fehler",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }

            this.CloseDeviceConnection();

            this.btnStartDizeRoll.Enabled = true;
            this.cbCOM.Enabled = true;
        }

        /// <summary>
        /// Closes the COM connection to the MMM8x8.
        /// </summary>
        private void CloseDeviceConnection()
        {
            if (this.device != null)
            {
                this.device.Close();
                this.device = null;
            }
        }
    }
}