﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows;
using System.Threading;

using System.Windows.Forms;

namespace ID100Snake
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// ID100 as ComPort-Device.
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// Number of the selected ComPort-Device
        /// </summary>
        private string comPortNumber;

        /// <summary>
        /// List of points, which represent the snake
        /// </summary>
        private List<Point> snake;

        /// <summary>
        /// Point which represents the food for the snake
        /// </summary>
        private Point food;

        /// <summary>
        /// Moving direction of the snake
        /// </summary>
        private int direction = 2;

        /// <summary>
        /// Indicates the status of the game
        /// </summary>
        private bool gamePaused = false;

        /// <summary>
        /// Indicates the status of the device connection
        /// </summary>
        private bool connected = false;

        /// <summary>
        /// Represents the game score
        /// </summary>
        private int score = 0;

        /// <summary>
        /// Indicates collision with border
        /// </summary>
        private bool hasCollided = false;

        public Form1()
        {
            InitializeComponent();
            this.AddAvailableComPorts();
            this.AddDegreeOfDifficulty();
            this.snake = new List<Point>();
            KeyPreview = true;
        }

        /// <summary>
        /// Adds numbers of available ComPort-devices to comboBox1
        /// </summary>
        private void AddAvailableComPorts()
        {
            foreach (string comport in SerialPort.GetPortNames())
            {
                this.comboBox1.Items.Add(comport);
                this.comboBox1.SelectedItem = comport;
            }
        }

        /// <summary>
        /// Adds the degrees of difficulty to comboBox2
        /// </summary>
        private void AddDegreeOfDifficulty()
        {
            List<string> degreesOfDifficulty = new List<string>();

            degreesOfDifficulty.Add("einfach");
            degreesOfDifficulty.Add("normal");
            degreesOfDifficulty.Add("schwierig");
            degreesOfDifficulty.Add("Experte");
            
            foreach (string degreeOfDifficulty in degreesOfDifficulty)
            {
                this.comboBox2.Items.Add(degreeOfDifficulty);   
                this.comboBox2.SelectedItem = degreesOfDifficulty[1];
            } 
        }


        /// <summary>
        /// Sets the degree of difficulty from comboBox2
        /// </summary>
        private void SetDegreeOfDifficulty()
        {
            switch (this.comboBox2.SelectedIndex)
            {
                case 0:
                    this.trigger.Interval = 550;
                    break;
                case 1:
                    this.trigger.Interval = 350;
                    break;
                case 2:
                    this.trigger.Interval = 200;
                    break;
                case 3:
                    this.trigger.Interval = 75;
                    break;
                default:
                    this.trigger.Interval = 350;
                    break;
            }
        }

        /// <summary>
        /// Connects to the wished ComPort.
        /// </summary>
        private ComPortDevice ConnectToComPort()
        {
            if (this.comboBox1.SelectedItem != null)
            {
                this.comPortNumber = this.comboBox1.SelectedItem.ToString();
            }

            SerialPort serialPort = new SerialPort(
                this.comPortNumber,
                38400,
                Parity.None,
                8,
                StopBits.One);

            try
            {
                serialPort.ReadTimeout = 500;
                serialPort.WriteTimeout = 500;
                serialPort.Open();
            }
            catch (UnauthorizedAccessException)
            {
                Console.WriteLine("FEHLER: Mit dem ComPort wurde bereits eine Verbindung hergestellt.");
            }
            catch (IOException)
            {
                Console.WriteLine("FEHLER: Mit dem ComPort ist keine Verbindung möglich.");
            }

            this.device = new ComPortDevice(serialPort);

            //Set the preview mode on connected ID100
            const byte SetPreviewModeCommand = (byte)'a';
            this.device.SendCommandToDevice(SetPreviewModeCommand, new byte[] { }, 0);
            return device;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (this.connected == false)
            {
                if (this.comboBox1.SelectedItem != null)
                {
                    this.comPortNumber = this.comboBox1.SelectedItem.ToString();
                }

                if (this.comPortNumber != null)
                {
                    this.ConnectToComPort();
                    this.ResetGame();
                    this.trigger.Start();
                    this.button1.Text = "ComPort-Verbindung trennen";
                    this.comboBox1.Enabled = false;
                    this.comboBox2.Enabled = false;
                    this.connected = true;
                }
                else
                {
                    MessageBox.Show(
                        "Es muss ein ComPort ausgewählt werden, um eine Verbindung herzustellen.",
                        "No ComPort selected",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Exclamation);
                }
            }
            else
            {
                this.trigger.Stop();
                this.device.SerialPort.Close();
                this.comboBox1.Enabled = true;
                this.comboBox2.Enabled = true;
                this.button1.Text = "ComPort-Verbindung herstellen";
                this.connected = false;
            }
        }

        /// <summary>
        /// Resets the game and creates the default snake
        /// </summary>
        private void ResetGame()
        {
            this.snake.Clear();
            this.snake.Add(new Point(0, 0));
            this.snake.Add(new Point(0, 1));
            this.snake.Add(new Point(0, 2));

            this.food = new Point(8, 6);
        }

        private void Trigger_Tick(object sender, EventArgs e)
        { 
            LedMatrixData ledMatrix = new LedMatrixData();

            foreach (Point snakePoint in this.snake)
            {
                ledMatrix.Leds[snakePoint.X + snakePoint.Y * 17] = true;
            }
          
            ledMatrix.Leds[this.food.X + this.food.Y * 17] = true;
            
            if (direction == 1)
            {
                this.MoveUp();
            }
            else if (direction == 2)
            {
                this.MoveDown();
            } 
            else if (direction == 3)
            {
                this.MoveLeft();
            }
            else if (direction == 4)
            {
                this.MoveRight();
            }

            this.CheckForFood();
            this.SetDegreeOfDifficulty();

            const byte SetAllLedsCommand = (byte)'D';
            this.device.SendCommandToDevice(SetAllLedsCommand, ledMatrix.GetDisplayData(), 0);
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyData)
            {
                case Keys.W:
                    if (direction != 2)
                    {
                        this.direction = 1;
                    }
                    break;
                case Keys.S:
                    if (direction != 1)
                    {
                        this.direction = 2;
                    }
                    break;
                case Keys.A:
                    if (direction != 4)
                    {
                        this.direction = 3;
                    }
                    break;
                case Keys.D:
                    if (direction != 3)
                    {
                        this.direction = 4;
                    }
                    break;
                case Keys.Escape:
                    if (gamePaused == false)
                    {
                        this.trigger.Stop();
                        gamePaused = true;
                        Console.WriteLine("Das Spiel wurde angehalten");
                    }
                    else
                    {
                        this.trigger.Start();
                        gamePaused = false;
                        Console.WriteLine("Das Spiel wird fortgesetzt");
                    }
                    break;
                default:
                    break;
            }
        }

        private void MoveUp()
        {
            int x = this.snake.Last<Point>().X;
            int y = this.snake.Last<Point>().Y;

            this.snake.RemoveAt(0);
            this.snake.Add(new Point(x, y - 1));
        }

        private void MoveDown()
        {
            int x = this.snake.Last<Point>().X;
            int y = this.snake.Last<Point>().Y;

            this.snake.RemoveAt(0);
            this.snake.Add(new Point(x, y + 1));
        }

        private void MoveLeft()
        {
            int x = this.snake.Last<Point>().X;
            int y = this.snake.Last<Point>().Y;

            this.snake.RemoveAt(0);
            this.snake.Add(new Point(x - 1, y));
        }

        private void MoveRight()
        {
            int x = this.snake.Last<Point>().X;
            int y = this.snake.Last<Point>().Y;

            this.snake.RemoveAt(0);
            this.snake.Add(new Point(x + 1, y));
        }

        private void CheckForFood()
        {
            int x = this.snake.Last<Point>().X;
            int y = this.snake.Last<Point>().Y;
            List<Point> feededSnake = new List<Point>();

            if (x == this.food.X && y == this.food.Y)
            {
                feededSnake.Insert(0, this.food);
                feededSnake.InsertRange(1, this.snake);
                
                this.snake = feededSnake;

                Random rX = new Random();
                Random rY = new Random();

                int xCoordinate = rX.Next(0, 16);
                int yCoordinate = rY.Next(0, 11);

                this.food = new Point(xCoordinate, yCoordinate);

                this.score += 10;
                this.label7.Text = score.ToString();
            }
        }

        private List<Point> CheckBorderCollision()
        {
            int x = this.snake.Last<Point>().X;
            int y = this.snake.Last<Point>().Y;

            if(x < 16) // Snake exits matrix on right
            {
                List<Point> snakeAfterBorderCollision = new List<Point>();

                foreach (Point snakePoint in this.snake)
                {
                    int index = this.snake.IndexOf(snakePoint);
                    snakeAfterBorderCollision.Insert(index, new Point(snakePoint.X, snakePoint.Y - 17));
                }
                this.hasCollided = true;
                return snakeAfterBorderCollision;
            }
            else if (x > 0) // Snake exits matrix on left 
            {
                List<Point> snakeAfterBorderCollision = new List<Point>();

                foreach (Point snakePoint in this.snake)
                {
                    int index = this.snake.IndexOf(snakePoint);
                    snakeAfterBorderCollision.Insert(index, new Point(snakePoint.X, snakePoint.Y + 17));
                }
                this.hasCollided = true;
                return snakeAfterBorderCollision;
            }
            else if (y > 0) // Snake exits matrix on top
            {
                List<Point> snakeAfterBorderCollision = new List<Point>();

                foreach (Point snakePoint in this.snake)
                {
                    int index = this.snake.IndexOf(snakePoint);
                    snakeAfterBorderCollision.Insert(index, new Point(snakePoint.X, snakePoint.Y + 11));
                }
                this.hasCollided = true;
                return snakeAfterBorderCollision;
            }
            else if (x < 16) // Snake exits matrix on buttom
            {
                List<Point> snakeAfterBorderCollision = new List<Point>();

                foreach (Point snakePoint in this.snake)
                {
                    int index = this.snake.IndexOf(snakePoint);
                    snakeAfterBorderCollision.Insert(index, new Point(snakePoint.X, snakePoint.Y - 11));
                }
                this.hasCollided = true;
                return snakeAfterBorderCollision;
            }
            else
            {
                this.hasCollided = false;
                return this.snake;
            }
        }
    }
}