﻿//-----------------------------------------------------------------------------
// <copyright file="Game.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Abstract class as prototype for a game.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Snake
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using MMM8x8Utils;

    /// <summary>
    /// Abstract class as prototype for a game.
    /// </summary>
    public abstract class Game
    {
        /// <summary>
        /// Initializes a new instance of the Game class.
        /// </summary>
        /// <param name="device">Needed ComPortDevice for visualization.</param>
        public Game(ComPortDevice device)
        {
        }

        /// <summary>
        /// EventHandler for advising clients if the score has changed.
        /// </summary>
        public abstract event EventHandler<ScoreEvent> ScoreChanged;

        /// <summary>
        /// Refreshing intervall of the game depending on the difficulty level.
        /// </summary>
        public enum Difficulty
        {
            /// <summary>
            /// Intervall in ms for easy difficulty.
            /// </summary>
            Easy = 240,

            /// <summary>
            /// Intervall in ms for normal difficulty.
            /// </summary>
            Normal = 160,

            /// <summary>
            /// Intervall in ms for hard difficulty.
            /// </summary>
            Hard = 80,

            /// <summary>
            /// Intervall in ms for expert difficulty.
            /// </summary>
            Expert = 40
        }

        /// <summary>
        /// Wished move direction for controlling the game.
        /// </summary>
        public enum MoveDirection
        {
            /// <summary>
            /// Move game object up.
            /// </summary>
            Up,

            /// <summary>
            /// Move game object down.
            /// </summary>
            Down,

            /// <summary>
            /// Move game object right.
            /// </summary>
            Right,

            /// <summary>
            /// Move game object left.
            /// </summary>
            Left
        }

        /// <summary>
        /// Current State of the game.
        /// </summary>
        public enum GameState
        {
            /// <summary>
            /// The game is currently running.
            /// </summary>
            Running,

            /// <summary>
            /// The player lost and the games ended.
            /// </summary>
            Ended
        }

        /// <summary>
        /// Gets or sets game difficulty.
        /// </summary>
        /// <value>Game difficulty.</value>
        public virtual Game.Difficulty GameDifficulty { get; protected set; }

        /// <summary>
        /// Gets or sets current state of the game.
        /// </summary>
        /// <value>Current state of the game.</value>
        public virtual Game.GameState CurrentState { get; set; }

        /// <summary>
        /// Gets or sets current moving direction.
        /// </summary>
        /// <value>Current moving direction.</value>
        public virtual Game.MoveDirection CurrentMovingDirection { get; set; }

        /// <summary>
        /// Calculate a new tick / single step of the game.
        /// </summary>
        public abstract void Tick();
    }
}
