﻿//-----------------------------------------------------------------------------
// <copyright file="ID100LedMatrix.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Represents the led matrix of the ID100.</summary>
//-----------------------------------------------------------------------------
namespace ID100Games
{
    using System;

    /// <summary>
    /// Represents the led matrix of the ID100.
    /// </summary>
    public class ID100LedMatrix
    {
        /// <summary>
        /// Led width of the ID100.
        /// </summary>
        public const int FieldWidth = 17;

        /// <summary>
        /// Led height of the ID100.
        /// </summary>
        public const int FieldHeight = 12;

        /// <summary>
        /// Amount of leds of the ID100.
        /// </summary>
        public const int AllLeds = ID100LedMatrix.FieldWidth * ID100LedMatrix.FieldHeight;

        /// <summary>
        /// Opcode for set all leds of the ID100.
        /// </summary>
        public const byte SetAllLedsCommand = (byte)'D';

        /// <summary>
        /// Opcode for enabling the preview mode of the ID100.
        /// </summary>
        public const byte SetPreviewModeCommand = (byte)'a';

        /// <summary>
        /// Leds of the ID100 as two-dimensional bool array.
        /// </summary>
        private bool[,] leds;

        /// <summary>
        /// Initializes a new instance of the ID100LedMatrix class.
        /// </summary>
        public ID100LedMatrix()
        {
            this.leds = new bool[ID100LedMatrix.FieldWidth, ID100LedMatrix.FieldHeight];
        }

        /// <summary>
        /// Gets or sets single leds of the ID100.
        /// </summary>
        /// <param name="x">X-coordinate of the led.</param>
        /// <param name="y">Y-coordinate of the led.</param>
        /// <returns>State of the wished led.</returns>
        public bool this[int x, int y]
        {
            get { return this.leds[x, y]; }
            set { this.leds[x, y] = value; }
        }

        /// <summary>
        /// Get a byte array of the current matrix used for sent as command to the ID100.
        /// </summary>
        /// <returns>Byte array of the current matrix.</returns>
        public byte[] GetBytes()
        {
            const int AmountOfBitsEachByte = 8;
            const int AmountOfBytes = 26;

            byte[] result = new byte[AmountOfBytes];
            for (int x = 0; x < ID100LedMatrix.FieldWidth; x++)
            {
                for (int y = 0; y < ID100LedMatrix.FieldHeight; y++)
                {
                    int index = (x * ID100LedMatrix.FieldHeight) + y;
                    int byteIndex = index / AmountOfBitsEachByte;
                    int bitIndex = Math.Abs((index % AmountOfBitsEachByte) - (AmountOfBitsEachByte - 1));

                    if (this.leds[x, y])
                    {
                        result[byteIndex] = ID100LedMatrix.SetBit(result[byteIndex], bitIndex);
                    }
                }
            }

            return result;
        }

        /// <summary>
        /// Sets all the leds of the ID100 with a specific state.
        /// </summary>
        /// <param name="state">Wished state.</param>
        public void SetAllLeds(bool state)
        {
            for (int x = 0; x < ID100LedMatrix.FieldWidth; x++)
            {
                for (int y = 0; y < ID100LedMatrix.FieldHeight; y++)
                {
                    this[x, y] = state;
                }
            }
        }

        /// <summary>
        /// Sets a specific bit of a byte.
        /// </summary>
        /// <param name="b">Used byte.</param>
        /// <param name="bitNumber">Number of the wished bit to set.</param>
        /// <returns>Edited byte.</returns>
        private static byte SetBit(byte b, int bitNumber)
        {
            byte result = b;
            if ((b & (byte)Math.Pow(2, bitNumber)) == 0)
            {
                result += (byte)Math.Pow(2, bitNumber);
            }

            return result;
        }
    }
}
