﻿//-----------------------------------------------------------------------------
// <copyright file="MainWindow.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Main window of the application.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Snake
{
    using System;
    using System.IO;
    using System.IO.Ports;
    using System.Threading;
    using System.Windows.Forms;
    using MMM8x8Utils;

    /// <summary>
    /// Main window of the application.
    /// </summary>
    public partial class MainWindow : Form
    {
        /// <summary>
        /// String to show if the game is not running.
        /// </summary>
        private const string StrStartGame = "starte Spiel";

        /// <summary>
        /// String to show if the game is running.
        /// </summary>
        private const string StrEndGame = "beende Spiel";

        /// <summary>
        /// List of the currently available ComPorts. 
        /// </summary>
        private string[] currentAvailableComPorts;

        /// <summary>
        /// Flag which indicates whether the program is connected to a COM or not.
        /// </summary>
        private bool connectedToComPort;

        /// <summary>
        /// Executed game.
        /// </summary>
        private Game game;

        /// <summary>
        /// MMM8x8 as ComPort-Device.
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            this.InitializeComponent();
            this.connectedToComPort = false;
        }

        /// <summary>
        /// Load-method of the main window.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void MainWindow_Load(object sender, EventArgs e)
        {
            this.AddAllTheAvailableComPorts();
            this.btnStartEndGame.Text = MainWindow.StrStartGame;

            this.cbGetComPort.SelectedIndex = 0;
            this.cbDifficulty.SelectedIndex = 0;
        }

        /// <summary>
        /// Adds all currently available ComPorts to the GUI.
        /// </summary>
        private void AddAllTheAvailableComPorts()
        {
            this.cbGetComPort.Items.Clear();

            this.currentAvailableComPorts = SerialPort.GetPortNames();
            this.cbGetComPort.Items.AddRange(this.currentAvailableComPorts);
        }

        /// <summary>
        /// Shows the score of the game in the GUI.
        /// </summary>
        /// <param name="score">Score of the game.</param>
        private void ShowScoreInGui(int score)
        {
            this.lbScore.Text = score.ToString();
        }

        /// <summary>
        /// Toggles state of the game.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void BtnStartEndGame(object sender, EventArgs e)
        {
            if (this.connectedToComPort)
            {
                this.StopGame();
            }
            else
            {
                this.StartGame();
            }
        }

        /// <summary>
        /// Starts the chosen game.
        /// </summary>
        private void StartGame()
        {
            const int StartPoints = 0;

            try
            {
                this.device = this.ConnectToCOM();
                this.SetPreviewModeOnDevice();
            }
            catch (InvalidOperationException ex)
            {
                MessageBox.Show(
                    ex.Message,
                    "Fehler",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            this.game = new SnakeGame(this.device);
            
            this.game.ScoreChanged += new EventHandler<ScoreEvent>(this.GameScore_Changed);
            this.ShowScoreInGui(StartPoints);

            // do ui stuff
            this.btnStartEndGame.Text = MainWindow.StrEndGame;
            this.EnableDisableSettingsGui(false);
            this.connectedToComPort = true;

            this.InitTimerSnakeTrigger();
        }

        /// <summary>
        /// Sets the preview mode on the MMM8x8.
        /// </summary>
        private void SetPreviewModeOnDevice()
        {
            this.device.SendCommandToDevice((byte)MMM8x8.Commands.SetPreviewModeCommand, new byte[] { });
        }

        /// <summary>
        /// Initialises the timer for triggering the ticks of the game.
        /// </summary>
        private void InitTimerSnakeTrigger()
        {
            Game.Difficulty difficulty;
            switch (this.cbDifficulty.SelectedIndex)
            {
                case 0:
                    difficulty = Game.Difficulty.Easy;
                    break;
                case 1:
                    difficulty = Game.Difficulty.Normal;
                    break;
                case 2:
                    difficulty = Game.Difficulty.Hard;
                    break;
                case 3:
                    difficulty = Game.Difficulty.Expert;
                    break;
                default:
                    throw new ArgumentException();
            }

            this.timerSnakeTrigger.Interval = (int)difficulty;
            this.timerSnakeTrigger.Start();
        }

        /// <summary>
        /// Stops the chosen game.
        /// </summary>
        private void StopGame()
        {
            // TODO Timer join...
            Thread.Sleep(500);

            this.timerSnakeTrigger.Stop();
            this.device.Close();

            // do ui stuff
            this.btnStartEndGame.Text = MainWindow.StrStartGame;
            this.EnableDisableSettingsGui(true);
            this.connectedToComPort = false;
        }

        /// <summary>
        /// Enables or disenables the GUI.
        /// </summary>
        /// <param name="enabledState">Wished state of the GUI.</param>
        private void EnableDisableSettingsGui(bool enabledState)
        {
            this.cbGetComPort.Enabled = enabledState;
            this.cbDifficulty.Enabled = enabledState;
        }

        /// <summary>
        /// Sets the text of the button to connect to a COM.
        /// </summary>
        /// <param name="text">Text of the button to connect to a COM.</param>
        private void SetConnectToComPortButtonText(string text)
        {
            this.btnStartEndGame.Text = text;
        }

        /// <summary>
        /// Connects to a chosen COM.
        /// </summary>
        /// <returns>COM-Wrapper for device communication.</returns>
        private ComPortDevice ConnectToCOM()
        {
            SerialPort serialPort = new SerialPort(
                this.GetChosenComNumber(),
                MMM8x8.BaudRate,
                MMM8x8.Parity,
                MMM8x8.Databits,
                MMM8x8.StopBits);

            try
            {
                serialPort.ReadTimeout = MMM8x8.ReadTimeout;
                serialPort.WriteTimeout = MMM8x8.WriteTimeout;
                serialPort.Open();
            }
            catch (UnauthorizedAccessException)
            {
                throw new InvalidOperationException("FEHLER: Mit dem COM wurde bereits eine Verbindung hergestellt.");
            }
            catch (IOException)
            {
                throw new InvalidOperationException("FEHLER: Mit dem COM ist keine Verbindung möglich.");
            }

            this.device = new ComPortDevice(serialPort);
            return this.device;
        }

        /// <summary>
        /// Gets the chosen index for connecting to a COM.
        /// </summary>
        /// <returns>Chosen index.</returns>
        private string GetChosenComNumber()
        {
            int selectedIndex = this.cbGetComPort.SelectedIndex;
            if (selectedIndex < 0)
            {
                throw new InvalidOperationException();
            }

            return this.currentAvailableComPorts[selectedIndex];
        }

        /// <summary>
        /// Tick-method of the timer which triggers the ticks of the game.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void TimerGameTrigger_Tick(object sender, EventArgs e)
        {
            this.game.Tick();

            if (this.game.CurrentState == Game.GameState.Ended)
            {
                this.StopGame();
            }
        }

        /// <summary>
        /// KeyDown-method of the main window.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            if (this.game == null)
            {
                return;
            }

            switch (e.KeyData)
            {
                case Keys.W:
                    this.game.CurrentMovingDirection = Game.MoveDirection.Up;
                    break;

                case Keys.S:
                    this.game.CurrentMovingDirection = Game.MoveDirection.Down;
                    break;

                case Keys.A:
                    this.game.CurrentMovingDirection = Game.MoveDirection.Left;
                    break;

                case Keys.D:
                    this.game.CurrentMovingDirection = Game.MoveDirection.Right;
                    break;

                default:
                    return;
            }
        }

        /// <summary>
        /// Receivs ScoreEvents from the game and refreshes the score in the GUI.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GameScore_Changed(object sender, ScoreEvent e)
        {
            this.ShowScoreInGui(e.Score);
        }
    }
}