﻿//-----------------------------------------------------------------------------
// <copyright file="SnakeGame.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Snake game for the MMM8x8.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Snake
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using MMM8x8Utils;

    /// <summary>
    /// Snake game for the MMM8x8.
    /// </summary>
    public class SnakeGame : Game
    {
        /// <summary>
        /// Start length of the snake.
        /// </summary>
        public const int StartLength = 3;

        /// <summary>
        /// Needed ComPortDevice for visualization.
        /// </summary>
        private ComPortDevice device;

        /// <summary>
        /// List of Point as representation of the snake.
        /// </summary>
        private List<Point> snake;

        /// <summary>
        /// Current position of the food which has to get collected.
        /// </summary>
        private Point food;

        /// <summary>
        /// Last direction of the snake.
        /// It is needed for checking whether a wished next moving direction is allowed or not.
        /// For example, the snake is not allowed to move down if it is currently moving up.
        /// </summary>
        private Game.MoveDirection lastDirection;

        /// <summary>
        /// Next direction of the snake (see above).
        /// </summary>
        private Game.MoveDirection nextDirection;

        /// <summary>
        /// Initializes a new instance of the SnakeGame class.
        /// </summary>
        /// <param name="device">Needed ComPortDevice for visualization.</param>
        public SnakeGame(ComPortDevice device) : base(device)
        {
            this.device = device;

            this.InitSnake();
        }

        /// <summary>
        /// EventHandler for advising clients if the score has changed.
        /// </summary>
        public override event EventHandler<ScoreEvent> ScoreChanged;

        /// <summary>
        /// Gets or sets current moving direction.
        /// </summary>
        /// <value>Current moving direction.</value>
        public override Game.MoveDirection CurrentMovingDirection
        {
            get
            {
                return this.nextDirection;
            }

            set
            {
                switch (value)
                {
                    case Game.MoveDirection.Up:
                        if (this.lastDirection != Game.MoveDirection.Down)
                        {
                            this.nextDirection = value;
                        }

                        break;

                    case Game.MoveDirection.Down:
                        if (this.lastDirection != Game.MoveDirection.Up)
                        {
                            this.nextDirection = value;
                        }

                        break;

                    case Game.MoveDirection.Right:
                        if (this.lastDirection != Game.MoveDirection.Left)
                        {
                            this.nextDirection = value;
                        }

                        break;

                    case Game.MoveDirection.Left:
                        if (this.lastDirection != Game.MoveDirection.Right)
                        {
                            this.nextDirection = value;
                        }

                        break;

                    default:
                        throw new ArgumentException();
                }
            }
        }

        /// <summary>
        /// Calculate a new tick / single step of the game.
        /// </summary>
        public override void Tick()
        {
            this.MoveSnake();

            if (this.CurrentState != GameState.Ended)
            {
                this.ShowCurrentSnakeOnDevice();
            }
        }

        /// <summary>
        /// Calculates and refreshes the current score.
        /// </summary>
        private void RefreshScore()
        {
            const int PointMultiplier = 10;

            int score = (this.snake.Count - SnakeGame.StartLength) * PointMultiplier;
            if (this.ScoreChanged != null)
            {
                this.ScoreChanged(this, new ScoreEvent(score));
            }
        }

        /// <summary>
        /// Initialises the snake.
        /// </summary>
        private void InitSnake()
        {
            this.snake = new List<Point>();
            this.CurrentMovingDirection = Game.MoveDirection.Down;

            for (int i = SnakeGame.StartLength - 1; i >= 0; i--)
            {
                this.snake.Add(new Point(MMM8x8LedMatrix.FieldWidth / 2, i));
            }

            this.SetRandomFood();
            this.ShowCurrentSnakeOnDevice();

            this.CurrentState = Game.GameState.Running;
            this.lastDirection = Game.MoveDirection.Right;
            this.nextDirection = Game.MoveDirection.Down;
        }

        /// <summary>
        /// Moves the snake according to the wished next direction.
        /// </summary>
        private void MoveSnake()
        {
            int dx, dy;

            switch (this.CurrentMovingDirection)
            {
                case Game.MoveDirection.Up:
                    dx = -1;
                    dy = 0;
                    break;

                case Game.MoveDirection.Down:
                    dx = 1;
                    dy = 0;
                    break;

                case Game.MoveDirection.Right:
                    dx = 0;
                    dy = 1;
                    break;

                case Game.MoveDirection.Left:
                    dx = 0;
                    dy = -1;
                    break;

                default:
                    throw new ArgumentException();
            }

            // remove last member of snake
            Point removedPoint = this.snake[this.snake.Count - 1];
            this.snake.RemoveAt(this.snake.Count - 1);

            // add new member to snake
            this.snake.Insert(0, this.FixPointCoordinate(new Point(this.snake[0].X + dx, this.snake[0].Y + dy)));

            if (this.snake[0].X == this.food.X &&
                this.snake[0].Y == this.food.Y)
            {
                this.snake.Add(removedPoint);
                this.ActionsAfterSnakeAteFood();
            }

            this.lastDirection = this.CurrentMovingDirection;

            if (this.CurrentState == GameState.Running)
            {
                this.CheckPosition();
            }
        }

        /// <summary>
        /// Execute all following actions if the snake eats the food.
        /// </summary>
        private void ActionsAfterSnakeAteFood()
        {
            this.SetRandomFood();
            this.MoveSnake();

            this.RefreshScore();
        }

        /// <summary>
        /// Checks the position of the Point objects of the Snake.
        /// </summary>
        private void CheckPosition()
        {
            List<int> positions = new List<int>();
            foreach (var point in this.snake)
            {
                positions.Add((point.Y * MMM8x8LedMatrix.FieldWidth) + point.X);
            }

            positions.Sort();
            for (int i = 1; i < positions.Count; i++)
            {
                if (positions[i] == positions[i - 1])
                {
                    this.PlayerFailed();
                    break;
                }
            }
        }

        /// <summary>
        /// Fixes the coordinates of a single point if it is out of range.
        /// </summary>
        /// <param name="point">Point to check and fix.</param>
        /// <returns>Fixed point.</returns>
        private Point FixPointCoordinate(Point point)
        {
            if (point.X < 0)
            {
                point.X += MMM8x8LedMatrix.FieldWidth;
            }
            else if (point.X >= MMM8x8LedMatrix.FieldWidth)
            {
                point.X = point.X % MMM8x8LedMatrix.FieldWidth;
            }

            if (point.Y < 0)
            {
                point.Y += MMM8x8LedMatrix.FieldHeight;
            }
            else if (point.Y >= MMM8x8LedMatrix.FieldHeight)
            {
                point.Y = point.Y % MMM8x8LedMatrix.FieldHeight;
            }

            return point;
        }

        /// <summary>
        /// Sets the food on a pseudo random space on the led field.
        /// If the calculated is to near to the snake a new space gets calculated.
        /// </summary>
        private void SetRandomFood()
        {
            Random r = new Random();
            Point newFood = new Point();

            do
            {
                newFood.X = r.Next(0, MMM8x8LedMatrix.FieldWidth);
                newFood.Y = r.Next(0, MMM8x8LedMatrix.FieldHeight);
            }
            while (this.IsPointNearSnake(newFood));

            this.food = newFood;
        }

        /// <summary>
        /// Checks whether the point is to near to the Point objects of the snake or not.
        /// </summary>
        /// <param name="point">Point to check.</param>
        /// <returns>Result of the check.</returns>
        private bool IsPointNearSnake(Point point)
        {
            const int MinimumGap = 1;

            foreach (var snakePoint in this.snake)
            {
                if (Math.Abs(snakePoint.X - point.X) <= MinimumGap &&
                    Math.Abs(snakePoint.Y - point.Y) <= MinimumGap)
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Shows the current snake on the MMM8x8.
        /// </summary>
        private void ShowCurrentSnakeOnDevice()
        {
            MMM8x8LedMatrix matrix = new MMM8x8LedMatrix();

            // add snake
            foreach (var point in this.snake)
            {
                matrix[point.X, point.Y] = true;
            }

            // add food
            matrix[this.food.X, this.food.Y] = true;

            // send matrix to device
            this.device.SendCommandToDevice((byte)MMM8x8.Commands.SetAllLedsCommand, matrix.GetBytes());
        }

        /// <summary>
        /// The player gets punished for his inabilities.
        /// </summary>
        private void PlayerFailed()
        {
            const bool StateOfLedsAfterFailing = true;

            MMM8x8LedMatrix matrix = new MMM8x8LedMatrix();
            matrix.SetAllLeds(StateOfLedsAfterFailing);

            // send matrix to device
            this.device.SendCommandToDevice((byte)MMM8x8.Commands.SetAllLedsCommand, matrix.GetBytes());

            this.CurrentState = Game.GameState.Ended;
        }
    }
}