﻿//-----------------------------------------------------------------------------
// <copyright file="ComPortDevice.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Represents a wrapper for a device on a Com-Port with methods for sending and receiving commands.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Utils
{
    using System;
    using System.Collections.Generic;
    using System.IO.Ports;
    using System.Threading;

    /// <summary>
    /// Represents a wrapper for a device on a Com-Port with methods for sending and receiving commands.
    /// </summary>
    public class ComPortDevice
    {
        /// <summary>
        /// Constant for byte checking in method EnqSpecialBytes.
        /// </summary>
        private const byte STX = 0x02;

        /// <summary>
        /// Constant for byte checking in method EnqSpecialBytes.
        /// </summary>
        private const byte ENQ = 0x10;

        /// <summary>
        /// Gets or sets used SerialPort of the device.
        /// </summary>
        /// <value>SerialPort of the device.</value>
        private SerialPort serialPort;

        /// <summary>
        /// Initializes a new instance of the ComPortDevice class.
        /// </summary>
        /// <param name="serialPort">Com-Port of the wrapper.</param>
        public ComPortDevice(SerialPort serialPort)
        {
            this.serialPort = serialPort;
        }
        
        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="opcode">Command opcode.</param>
        /// <param name="para">Command parameters.</param>
        /// <returns>Answer of the device.</returns>
        public byte[] SendCommandToDevice(byte opcode, byte[] para)
        {
            List<byte> buffer = new List<byte>();
            List<byte> dataFrame = new List<byte>();

            dataFrame.Add(opcode);
            dataFrame.AddRange(para);

            byte[] header = new byte[2];
            header[0] = (byte)((dataFrame.ToArray().Length >> 8) & 0xff);
            header[1] = (byte)(dataFrame.ToArray().Length & 0xff);
            header = this.EnqSpecialBytes(header); // Escape special chars from the header
            dataFrame = new List<byte>(this.EnqSpecialBytes(dataFrame.ToArray())); // Escape special chars from the dataframe

            // Calculate CRC16 for the startchar, the header and the dataframe
            CRC16 crc = new CRC16();
            crc.Shift(STX);
            crc.Shift(header);
            crc.Shift(dataFrame.ToArray());

            // Combine all data parts for transfering to the device
            List<byte> output = new List<byte>();
            output.Add(STX);
            output.AddRange(header);
            output.AddRange(dataFrame);
            output.AddRange(this.EnqSpecialBytes(crc.CrcBytes)); // Escape special chars from the crc

            int bytesWriteRequest = output.Count;
            this.serialPort.Write(output.ToArray(), 0, bytesWriteRequest);

            Thread.Sleep(50);

            var readBuffer = new byte[200];
            this.serialPort.Read(readBuffer, 0, readBuffer.Length);
            return this.ParseReadAnswer(opcode, readBuffer);
        }

        /// <summary>
        /// Closes the connection to the COM.
        /// </summary>
        public void Close()
        {
            if (this.serialPort != null && this.serialPort.IsOpen)
            {
                this.serialPort.Close();
            }
        }

        /// <summary>
        /// Gets the Answer of a transmitted command from the device.
        /// </summary>
        /// <param name="expectedOpcode">Opcode of the transmitted command (needed for verification).</param>
        /// <param name="amountToRead">Amount of bytes which are read for the answer.</param>
        /// <returns>Answer of the transmitted command from the device.</returns>
        private byte[] ReadAnswerAfterCommand(byte expectedOpcode, int amountToRead)
        {
            /*
             * Aufbau der ausgelesenen Daten
             * 1 Byte: Startzeichen (STX)
             * 2 Byte: Länge der folgenden Daten
             * siehe Länge: ausgelesene Daten mit Opcode
             * 2 Byte: CRC16
             */
            const int READ_OFFSET = 5;
            byte[] readData = new byte[amountToRead + READ_OFFSET];

            try
            {
                this.serialPort.Read(readData, 0, amountToRead + READ_OFFSET);

                if (readData[0] == 0x02)
                {
                    readData = this.DeenqSpecialBytes(readData);
                    byte readOpcode = readData[2];
                    if (readOpcode != expectedOpcode)
                    {
                        return new byte[0];
                    }

                    return readData;
                }
                else
                {
                    return new byte[0];
                }
            }
            catch
            {
                throw new InvalidOperationException("Mit dem ausgewählten COM ist keine Verbindung möglich.");
            }
        }

        /// <summary>
        /// Converts special bytes before transmitting the command.
        /// </summary>
        /// <param name="buffer">Command for the device.</param>
        /// <returns>Converted command.</returns>
        private byte[] EnqSpecialBytes(byte[] buffer)
        {
            List<byte> temp = new List<byte>();

            for (int i = 0; i < buffer.Length; i++)
            {
                if (buffer[i] == STX)
                {
                    temp.Add(ENQ);
                    temp.Add(STX + 0x80);
                }
                else if (buffer[i] == ENQ)
                {
                    temp.Add(ENQ);
                    temp.Add(ENQ + 0x80);
                }
                else
                {
                    temp.Add(buffer[i]);
                }
            }

            return temp.ToArray();
        }

        /// <summary>
        /// Removes escape chars from the received data.
        /// </summary>
        /// <param name="readData">Readed data from device.</param>
        /// <returns>Data from command.</returns>
        private byte[] DeenqSpecialBytes(byte[] readData)
        {
            List<byte> temp = new List<byte>();
            bool escaped = false;

            for (int i = 0; i < readData.Length; i++)
            {
                if (readData[i] == STX)
                {
                }
                else if (readData[i] == ENQ)
                {
                    escaped = true;
                }
                else if (escaped)
                {
                    temp.Add((byte)(readData[i] & 0x7F));
                    escaped = false;
                }
                else
                {
                    temp.Add(readData[i]);
                }
            }

            return temp.ToArray();
        }

        /// <summary>
        /// Parses and verifies the read answer.
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <param name="readAnswer">Read answer.</param>
        /// <returns>Filtered payload.</returns>
        private byte[] ParseReadAnswer(byte opcode, byte[] readAnswer)
        {
            const byte EmptyData = 0x00;
            const int OpcodeIndex = 2;

            var data = new List<byte>(this.DeenqSpecialBytes(readAnswer));

            byte opcodeReadData = data[OpcodeIndex];
            if (opcodeReadData != opcode)
            {
                throw new InvalidOperationException(string.Format(
                    "The opcodes of send and received messages do not match (send: 0x{0}, received: 0x{1}).",
                    ((byte)opcode).ToString("X2"),
                    opcodeReadData.ToString("X2")));
            }

            // remove empty data
            while (data[data.Count - 1] == EmptyData)
            {
                data.RemoveAt(data.Count - 1);
            }

            // remove crc
            data.RemoveAt(data.Count - 1);
            data.RemoveAt(data.Count - 1);

            // remove first three bytes (STX, payload length)
            data.RemoveAt(2);
            data.RemoveAt(1);
            data.RemoveAt(0);

            return data.ToArray();
        }
    }
}