﻿//-----------------------------------------------------------------------------
// <copyright file="MMM8x8LedMatrix.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2014 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Represents the led matrix of the MMM8x8.</summary>
//-----------------------------------------------------------------------------
namespace MMM8x8Snake
{
    using System;

    /// <summary>
    /// Represents the led matrix of the MMM8x8.
    /// </summary>
    public class MMM8x8LedMatrix
    {
        /// <summary>
        /// Amount of leds in the width.
        /// </summary>
        public const int FieldWidth = 8;

        /// <summary>
        /// Amount of leds in the height.
        /// </summary>
        public const int FieldHeight = 8;

        /// <summary>
        /// Amount of all leds.
        /// </summary>
        public const int AllLeds = MMM8x8LedMatrix.FieldWidth * MMM8x8LedMatrix.FieldHeight;

        /// <summary>
        /// Leds of the MMM8x8 as two-dimensional bool array.
        /// </summary>
        private bool[,] leds;

        /// <summary>
        /// Initializes a new instance of the MMM8x8LedMatrix class.
        /// </summary>
        public MMM8x8LedMatrix()
        {
            this.leds = new bool[MMM8x8LedMatrix.FieldWidth, MMM8x8LedMatrix.FieldHeight];
        }

        /// <summary>
        /// Gets or sets a single led of the MMM8x8.
        /// LED[0, 0] = on top left
        /// LED[0, 7] = on top right
        /// LED[7, 0] = on bottom left
        /// LED[7, 7] = on bottom right
        /// The USB-interface has to be located on the at the back (viewed from
        /// the front of the device).
        /// </summary>
        /// <param name="row">Row of the led.</param>
        /// <param name="column">Column of the led.</param>
        /// <returns>State of the wished led.</returns>
        public bool this[int row, int column]
        {
            get { return this.leds[row, column]; }
            set { this.leds[row, column] = value; }
        }

        /// <summary>
        /// Gets a byte array of the current matrix used to get sent as command.
        /// </summary>
        /// <returns>Byte array of the current matrix.</returns>
        public byte[] GetBytes()
        {
            const int AmountOfBytes = 8;
            
            byte[] result = new byte[AmountOfBytes];

            for (int row = 0; row < MMM8x8LedMatrix.FieldWidth; row++)
            {
                for (int column = 0; column < MMM8x8LedMatrix.FieldHeight; column++)
                {
                    MMM8x8LedMatrix.SetBit(ref result[column], row, this[row, column]);
                }
            }

            return result;
        }

        /// <summary>
        /// Sets all the leds with a specific state.
        /// </summary>
        /// <param name="state">Wished state.</param>
        public void SetAllLeds(bool state)
        {
            for (int row = 0; row < MMM8x8LedMatrix.FieldWidth; row++)
            {
                for (int column = 0; column < MMM8x8LedMatrix.FieldHeight; column++)
                {
                    this[row, column] = state;
                }
            }
        }

        /// <summary>
        /// Sets a specific bit of a byte.
        /// </summary>
        /// <param name="b">Used byte.</param>
        /// <param name="index">Index of the wanted bit.</param>
        /// <param name="state">Wished state of the wanted bit.</param>
        private static void SetBit(ref byte b, int index, bool state)
        {
            byte add = (byte)Math.Pow(2, index);
            if ((b & add) != 0)
            {
                if (!state)
                {
                    b -= add;
                }
            }
            else
            {
                if (state)
                {
                    b += add;
                }
            }
        }
    }
}
